package solutions.bitbadger.documents

import java.sql.Connection
import java.sql.ResultSet

/**
 * Execute a query that returns a list of results
 *
 * @param query The query to retrieve the results
 * @param parameters Parameters to use for the query
 * @param mapFunc The mapping function between the document and the domain item
 * @return A list of results for the given query
 */
inline fun <reified TDoc> Connection.customList(
    query: String, parameters: Collection<Parameter<*>>, mapFunc: (ResultSet) -> TDoc
) = Custom.list(query, parameters, this, mapFunc)

/**
 * Execute a query that returns one or no results
 *
 * @param query The query to retrieve the results
 * @param parameters Parameters to use for the query
 * @param mapFunc The mapping function between the document and the domain item
 * @return The document if one matches the query, `null` otherwise
 */
inline fun <reified TDoc> Connection.customSingle(
    query: String, parameters: Collection<Parameter<*>>, mapFunc: (ResultSet) -> TDoc
) = Custom.single(query, parameters, this, mapFunc)

/**
 * Execute a query that returns no results
 *
 * @param query The query to retrieve the results
 * @param parameters Parameters to use for the query
 */
fun Connection.customNonQuery(query: String, parameters: Collection<Parameter<*>>) =
    Custom.nonQuery(query, parameters, this)

/**
 * Execute a query that returns a scalar result
 *
 * @param query The query to retrieve the result
 * @param parameters Parameters to use for the query
 * @param mapFunc The mapping function between the document and the domain item
 * @return The scalar value from the query
 */
inline fun <reified T> Connection.customScalar(
    query: String,
    parameters: Collection<Parameter<*>>,
    mapFunc: (ResultSet) -> T & Any
) = Custom.scalar(query, parameters, this, mapFunc)
