package solutions.bitbadger.documents

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.assertThrows
import kotlin.test.assertEquals
import kotlin.test.assertFalse
import kotlin.test.assertTrue

/**
 * Unit tests for the `Configuration` object
 */
@DisplayName("Configuration")
class ConfigurationTest {

    @Test
    @DisplayName("Default JSON options are as expected")
    fun defaultJsonOptions() {
        assertTrue(Configuration.json.configuration.encodeDefaults, "Encode Defaults should have been set")
        assertFalse(Configuration.json.configuration.explicitNulls, "Explicit Nulls should not have been set")
    }

    @Test
    @DisplayName("Default ID field is `id`")
    fun defaultIdField() {
        assertEquals("id", Configuration.idField, "Default ID field incorrect")
    }

    @Test
    @DisplayName("Default Auto ID strategy is `DISABLED`")
    fun defaultAutoId() {
        assertEquals(AutoId.DISABLED, Configuration.autoIdStrategy, "Default Auto ID strategy should be `disabled`")
    }

    @Test
    @DisplayName("Default ID string length should be 16")
    fun defaultIdStringLength() {
        assertEquals(16, Configuration.idStringLength, "Default ID string length should be 16")
    }

    @Test
    @DisplayName("Dialect is derived from connection string")
    fun dialectIsDerived() {
        try {
            assertThrows<DocumentException> { Configuration.dialect() }
            Configuration.connectionString = "jdbc:postgresql:db"
            assertEquals(Dialect.POSTGRESQL, Configuration.dialect())
        } finally {
            Configuration.connectionString = null
        }
    }
}
